/**
 * @copyright (c) Solution 360 GmbH
 * Employs floating labels for textual input fields.
 *
 * Relies on the structure:
 *
 * div.float-label-control
 *  label
 *  input
 */
(function ($) {

    function s360_floatlabel_init() {
        $('.float-label-control:not(.s360-float-label-initialized)').each(function () {
            var $this = $(this);
            $this.addClass('s360-float-label-initialized');
            var $label = $this.find('label');
            if (!$label.length) {
                return this;
            }

            var $input = $this.find('#' + $label.attr('for'));
            if ($input.is('select')) {
                // hide all labels on select fields
                $label.hide();
                return this;
            }

            s360_floatlabel_update($input, $label);
            $input.on('focus blur change keyup', function () {
                var $this = $(this);
                s360_floatlabel_update($this, $this.closest('.float-label-control').find('label[for="' + $this.attr('id') + '"]'));
            });
        });
    }

    function s360_floatlabel_update($input, $label) {
        if ($input.is(':focus') || (typeof $input.val() !== "undefined" && $input.val() !== '') || typeof $input.attr('placeholder') === "undefined" || $input.attr('placeholder') === '') {
            // display floating label
            $label.show();
            $label.addClass('visible');
        } else {
            // hide floating label, set placeholder to label value if it doesn't exist
            if (typeof $input.attr('placeholder') === "undefined" || $input.attr('placeholder') === '') {
                $input.attr('placeholder', $label.text());
            }
            $label.hide();
            $label.removeClass('visible');
        }
    }

    // CALLS
    // ===============
    $(document).ready(function () {
        s360_floatlabel_init();

        $('body').on('article-variation-loaded.s360', _.debounce(function () {
            s360_floatlabel_init();
        }, 200));
    });
})(jQuery);

