/**
 * @copyright (c) Solution 360 GmbH
 * Functions for the main menu.
 */
(function ($) {

    function s360_adjust_main_menu() {
        var $mainMenu = $('.s360-main-menu:not(.s360-main-menu-overflow)');
        var $overflowContainer = $('#s360-main-menu-overflow');
        var $overflowToggle = $('.s360-main-menu-overflow-toggle');

        // reset elements, i.e. move everything from the overflow back into the main menu to recalculate
        $overflowContainer.children().appendTo($mainMenu);
        $overflowContainer.html('');

        var mainMenuWidth = $mainMenu.outerWidth(false);

        // get toggle button width - for this purpose, briefly show the overflow-Toggle, to get its width
        $overflowToggle.removeClass('hidden');
        var toggleWidth = $overflowToggle.outerWidth(true) + 1; // add +1 because jQuery outerWidth returns floored ints
        $overflowToggle.addClass('hidden');

        var $menuElements = $mainMenu.find('.s360-main-menu-item-wrapper:not(.s360-main-menu-overflow-toggle)');
        var menuElementsCount = $menuElements.length;
        var keep = true;
        var currentWidth = 0;

        $menuElements.each(function (index, element) {
            var $this = $(this);
            if (!keep) {
                // short cut, all remaining elements will be moved
                $this.appendTo($overflowContainer);
                return;
            }
            var elementWidth = $this.outerWidth(true) + 1; // add +1 because jQuery outerWidth returns floored ints
            if (index < (menuElementsCount - 1)) {
                if (currentWidth + toggleWidth + elementWidth > mainMenuWidth) {
                    keep = false;
                    $this.appendTo($overflowContainer);
                }
            } else {
                // last element, here the width *without* the toggle button needs to be considered
                if (currentWidth + elementWidth > mainMenuWidth) {
                    keep = false;
                    $this.appendTo($overflowContainer);
                }
            }
            currentWidth = currentWidth + elementWidth;
        });

        if ($overflowContainer.children().length) {
            $overflowToggle.removeClass('hidden');
        } else {
            $overflowToggle.addClass('hidden');
        }
    }

    function s360_adjust_vertical_scroll($menu) {
        // reset styles
        var scrollTopBefore = $menu.scrollTop();
        $menu.css({
            maxHeight: 'none',
            overflowY: 'hidden'
        });
        // only fix for sticky menus
        if ($menu.parents('.s360-sticky-active').length) {
            // check if the element is in the viewport completely
            var offsetTop = $menu.offset().top;
            var outerHeight = $menu.outerHeight(false);
            var $window = $(window);
            var windowHeight = $window.height();
            var windowScrollTop = $window.scrollTop();
            var realOffsetTop = offsetTop - windowScrollTop;
            if (realOffsetTop > 0 && realOffsetTop + outerHeight > windowHeight) {
                // element is not completely in the viewport, but is hanging on a sticky parent - we need to set a scroll bar
                var targetHeight = Math.floor(windowHeight - realOffsetTop);
                $menu.css({
                    maxHeight: targetHeight + 'px',
                    overflowY: 'scroll'
                });
                $menu.scrollTop(scrollTopBefore);
            }
        }
    }


    // CALLS
    // ===============
    $(window).load(function () {
        s360_adjust_main_menu();
        var debounced = _.debounce(function () {
            s360_adjust_main_menu();
        }, 16);
        $(window).resize(debounced);

        // vertical scrolling for overly long dropdowns in the menu
        $('.s360-main-menu').on('shown.bs.dropdown', '.dropdown', function () {
            s360_adjust_vertical_scroll($(this).closest('.dropdown').find('.dropdown-menu'));
        });
        var debouncedAdjustScroll = _.debounce(function (e) {
            var $openMenu = $('.s360-main-menu .dropdown.open .dropdown-menu');
            if ($openMenu.length) {
                s360_adjust_vertical_scroll($openMenu);
            }
        }, 16);
        $(window).scroll(function (e) {
            debouncedAdjustScroll(e);
        });
    });
})(jQuery);
