/* 
 * @copyright (c) Solution 360 GmbH
 * Multiple small utils
 */
(function ($) {

    function s360_init_sliders() {

        var mobileSpeed = 150; // default speed is 300ms, we want to be quicker on mobile
        var defaultSpeed = 300;
        // initiate slick sliders, reset evo sliders that might have been initialized by EVO JS
        /*
         * responsive slider (content, products)
         */
        $('.evo-slider.slick-intialized').slick('unslick');

        $('.evo-slider:not(.slick-initialized), .s360-slick-slider:not(.slick-initialized)').each(function(index, elem) {
            var $this = $(elem);
            // defaults
            var slideCount = {
                xs: 1,
                sm: 3,
                md: 3,
                lg: 4
            };
            // load custom settings if given
            if(typeof $this.data('slides-xs') !== "undefined" && $this.data('slides-xs') !== null) {
                slideCount.xs = $this.data('slides-xs');
            }
            if(typeof $this.data('slides-sm') !== "undefined" && $this.data('slides-sm') !== null) {
                slideCount.sm = $this.data('slides-sm');
            }
            if(typeof $this.data('slides-md') !== "undefined" && $this.data('slides-md') !== null) {
                slideCount.md = $this.data('slides-md');
            }
            if(typeof $this.data('slides-lg') !== "undefined" && $this.data('slides-lg') !== null) {
                slideCount.lg = $this.data('slides-lg');
            }
            var options = {
                arrows: true,
                infinite: true,
                lazyLoad: 'ondemand',
                mobileFirst: true,
                responsive: [
                    {
                        breakpoint: _s360.breakpoints.xs - 1, // xs
                        settings: {
                            slidesToShow: slideCount.xs,
                            slidesToScroll: slideCount.xs,
                            speed: mobileSpeed
                        }
                    },
                    {
                        breakpoint: _s360.breakpoints.sm - 1, // sm
                        settings: {
                            slidesToShow: slideCount.sm,
                            slidesToScroll: slideCount.sm,
                            speed: defaultSpeed
                        }
                    },
                    {
                        breakpoint: _s360.breakpoints.md - 1, // md
                        settings: {
                            slidesToShow: slideCount.md,
                            slidesToScroll: slideCount.md,
                            speed: defaultSpeed
                        }
                    },
                    {
                        breakpoint: _s360.breakpoints.lg - 1, // lg
                        settings: {
                            slidesToShow: slideCount.lg,
                            slidesToScroll: slideCount.lg,
                            speed: defaultSpeed
                        }
                    }
                ]
            };
            $this.slick(options);
        });

        /*
         * responsive slider (main)
         */
        $('.s360-image-slider-slick:not(.slick-initialized)').slick({
            mobileFirst: true,
            responsive: [
                {
                    breakpoint: _s360.breakpoints.xs - 1, // xs
                    settings: {
                        speed: mobileSpeed
                    }
                },
                {
                    breakpoint: _s360.breakpoints.sm - 1, // sm
                    settings: {
                        speed: defaultSpeed
                    }
                },
                {
                    breakpoint: _s360.breakpoints.md - 1, // md
                    settings: {
                        speed: defaultSpeed
                    }
                },
                {
                    breakpoint: _s360.breakpoints.lg - 1, // lg
                    settings: {
                        speed: defaultSpeed
                    }
                }
            ]
        });

        /*
         * brand sliders
         */
        $('.s360-brand-slider:not(.slick-initialized)').each(function (id, elem) {
            var $this = $(elem);
            // max 9 slides at once, but if we have fewer, distribute evenly
            var slidesToShowXS = Math.min($this.children().length, 2);
            var slidesToShowSM = Math.min($this.children().length, 4);
            var slidesToShowMD = Math.min($this.children().length, 6);
            var slidesToShowLG = Math.min($this.children().length, 9);
            $this.slick({
                dots: false,
                arrows: true,
                lazyLoad: 'ondemand',
                infinite: true,
                mobileFirst: true,
                responsive: [
                    {
                        breakpoint: _s360.breakpoints.xs - 1, // xs
                        settings: {
                            slidesToShow: slidesToShowXS,
                            slidesToScroll: 1,
                            speed: mobileSpeed
                        }
                    },
                    {
                        breakpoint: _s360.breakpoints.sm - 1, // sm
                        settings: {
                            slidesToShow: slidesToShowSM,
                            slidesToScroll: 1,
                            speed: defaultSpeed
                        }
                    },
                    {
                        breakpoint: _s360.breakpoints.md - 1, // md
                        settings: {
                            slidesToShow: slidesToShowMD,
                            slidesToScroll: 1,
                            speed: defaultSpeed
                        }
                    },
                    {
                        breakpoint: _s360.breakpoints.lg - 1, // lg
                        settings: {
                            slidesToShow: slidesToShowLG,
                            slidesToScroll: 1,
                            speed: defaultSpeed
                        }
                    }
                ]
            });
        });

        /*
         * Simple image slider
         */
        $('.s360-image-simple-slider:not(.slick-initialized)').each(function (id, elem) {
            var $this = $(elem);
            // max 9 slides at once, but if we have fewer, distribute evenly
            var slidesToShowXS = $this.data('slidesXs') ? $this.data('slidesXs') : 1;
            var slidesToShowSM = $this.data('slidesSm') ? $this.data('slidesSm') : 3;
            var slidesToShowMD = $this.data('slidesMd') ? $this.data('slidesMd') : 3;
            var slidesToShowLG = $this.data('slidesLg') ? $this.data('slidesLg') : 5;
            $this.slick({
                lazyLoad: 'ondemand',
                infinite: true,
                mobileFirst: true,
                responsive: [
                    {
                        breakpoint: _s360.breakpoints.xs - 1, // xs
                        settings: {
                            slidesToShow: slidesToShowXS,
                            slidesToScroll: 1,
                            speed: mobileSpeed
                        }
                    },
                    {
                        breakpoint: _s360.breakpoints.sm - 1, // sm
                        settings: {
                            slidesToShow: slidesToShowSM,
                            slidesToScroll: 1,
                            speed: defaultSpeed
                        }
                    },
                    {
                        breakpoint: _s360.breakpoints.md - 1, // md
                        settings: {
                            slidesToShow: slidesToShowMD,
                            slidesToScroll: 1,
                            speed: defaultSpeed
                        }
                    },
                    {
                        breakpoint: _s360.breakpoints.lg - 1, // lg
                        settings: {
                            slidesToShow: slidesToShowLG,
                            slidesToScroll: 1,
                            speed: defaultSpeed
                        }
                    }
                ]
            });
        });

        /*
         * Viewbox slider (quadruple viewboxes)
         */
        $('.s360-viewbox-slider:not(.slick-initialized), .s360-news-slider:not(.slick-initialized)').slick({
            mobileFirst: true,
            responsive: [
                {
                    breakpoint: _s360.breakpoints.xs - 1, // xs
                    settings: {
                        slidesToShow: 1,
                        slidesToScroll: 1,
                        speed: mobileSpeed
                    }
                },
                {
                    breakpoint: _s360.breakpoints.sm - 1, // sm
                    settings: {
                        slidesToShow: 3,
                        slidesToScroll: 3,
                        speed: defaultSpeed
                    }
                },
                {
                    breakpoint: _s360.breakpoints.md - 1, // md
                    settings: {
                        slidesToShow: 3,
                        slidesToScroll: 3,
                        speed: defaultSpeed
                    }
                },
                {
                    breakpoint: _s360.breakpoints.lg - 1, // lg
                    settings: {
                        slidesToShow: 4,
                        slidesToScroll: 4,
                        speed: defaultSpeed
                    }
                }
            ]
        });

        /*
         * Viewbox slider (triple viewboxes)
         */
        $('.s360-viewbox-slider-triple:not(.slick-initialized)').slick({
            mobileFirst: true,
            responsive: [
                {
                    breakpoint: _s360.breakpoints.xs - 1, // xs
                    settings: {
                        slidesToShow: 1,
                        slidesToScroll: 1,
                        speed: mobileSpeed
                    }
                },
                {
                    breakpoint: _s360.breakpoints.sm - 1, // sm
                    settings: {
                        slidesToShow: 3,
                        slidesToScroll: 3,
                        speed: defaultSpeed
                    }
                },
                {
                    breakpoint: _s360.breakpoints.md - 1, // md
                    settings: {
                        slidesToShow: 3,
                        slidesToScroll: 3,
                        speed: defaultSpeed
                    }
                },
                {
                    breakpoint: _s360.breakpoints.lg - 1, // lg
                    settings: {
                        slidesToShow: 3,
                        slidesToScroll: 3,
                        speed: defaultSpeed
                    }
                }
            ]
        });

        // initiate ion range slider
        $('.price-slider-input, .price-slider-input-jtl-search').each(function () {
            $slider = $(this);
            var $sliderParent = $slider.closest('.s360-filter-snippet');
            if ($slider.length && $sliderParent.length) {
                $slider.ionRangeSlider({
                    onInitFinished: function (slider) {
                        // initially update price fields
                        $sliderParent.find('.s360-price-slider-display-min').html($sliderParent.find('.irs-from').html());
                        $sliderParent.find('.s360-price-slider-display-max').html($sliderParent.find('.irs-to').html());
                    },
                    onChange: function (slider) {
                        // update price fields on slide change to show current value
                        $sliderParent.find('.s360-price-slider-display-min').html($sliderParent.find('.irs-from').html());
                        $sliderParent.find('.s360-price-slider-display-max').html($sliderParent.find('.irs-to').html());
                    },
                    onFinish: function (slider) {
                        // initiate redirect to price filter url
                        var priceFilterString = slider.from + "_" + slider.to;
                        var baseUrl = $slider.data('baseurl');
                        var anchor = $slider.data('anchor');
                        if($slider.hasClass('price-slider-input-jtl-search')) {
                            window.location.href = baseUrl + priceFilterString + anchor;
                        } else {
                            window.location.href = baseUrl.replace(/(\W)pf=(\d|\.)+_(\d|\.)+/, '$1pf=' + priceFilterString + anchor);
                        }
                    }
                });
            }
        });

    }

    $(document).ready(function () {
        // add left swipe function to mobile menu
        $('.s360-mobile-menu').on('swipeleft', function () {
            $('.s360-mobile-menu').find('.btn-close').click();
        });

        var touchCapable = isTouchCapable();

        if (touchCapable) {
            $('body').on('swipeleft', '.s360-filter-offcanvas.in', function () {
                $(document).click(); // emulate click on the document - offcanvas will react to this
            });
        }

        // make bootstrap dropdowns hoverable on non-touch devices
        // isTouchCapable is a new function in EVO 4.06's global.js
        if (!touchCapable) {
            // delay in milliseconds before hover takes effect
            var hoverIntentDelayInbound = 250;
            var hoverIntentDelayOutbound = 250;
            var showElementFunc = function ($elem) {
                $elem.data('hoverInTimer', null);
                if(!$elem.hasClass('open')) {
                    $elem.trigger('show.bs.dropdown');
                    $elem.addClass('open');
                    $elem.find('.dropdown-menu').stop(true, true).delay(100).fadeIn(250);
                    $elem.trigger('shown.bs.dropdown');
                }
            };
            var hideElementFunc = function($elem, e) {
                $elem.data('hoverOutTimer', null);
                var $menu = $elem.find('.dropdown-menu');
                var mouseX = e.pageX - $(window).scrollLeft();
                var mouseY = e.pageY - $(window).scrollTop();
                var myMinX = $menu.offset().left - $(window).scrollLeft();
                var myMinY = $menu.offset().top - $(window).scrollTop();
                var myMaxX = myMinX + $menu.outerWidth(true);
                var myMaxY = myMinY + $menu.outerHeight(true);
                if (mouseX >= myMinX && mouseX <= myMaxX && mouseY >= myMinY && mouseY <= myMaxY) {
                    // caught a false hover out event! do nothing!
                    return;
                }
                if($elem.hasClass('open')) {
                    $elem.trigger('hide.bs.dropdown');
                    $elem.removeClass('open');
                    $menu.stop(true, true).delay(100).fadeOut(250);
                    $elem.trigger('hidden.bs.dropdown');
                }
            };
            $('.dropdown:not(.dropdown-nohover)').hover(function () {
                var $this = $(this);
                var timerOut = $this.data("hoverOutTimer");
                if(timerOut) {
                    // hover out timer is running, clear it, to prevent hiding the menu (we just hovered back in)
                    clearTimeout(timerOut);
                    $this.data('hoverOutTimer', null);
                    showElementFunc($this);
                } else {
                    var timerIn = $this.data("hoverInTimer");
                    if (!timerIn) {
                        $this.data('hoverInTimer', setTimeout(function () {
                            showElementFunc($this);
                        }, hoverIntentDelayInbound));
                    }
                }
            }, function (e) {
                // we might need to handle browser popups for account selection - if these are currently hovered, we do not want to hide the dropdown!
                // this is the case when the cursors position is technically still over the dropdown
                var $this = $(this);
                var timerIn = $this.data("hoverInTimer");
                if(timerIn) {
                    // hover in timer is running, clear it, do nothing.
                    clearTimeout(timerIn);
                    $this.data('hoverInTimer', null);
                    hideElementFunc($this, e);
                } else {
                    var timerOut = $this.data("hoverOutTimer");
                    if(!timerOut) {
                        $this.data('hoverOutTimer', setTimeout(function() {
                            hideElementFunc($this, e);
                        }, hoverIntentDelayOutbound));
                    }
                }
            });
        }


        // make open bootstrap dropdown-toggle links clickable
        // this is an important preparation: we have to explicitly hide all dropdown-menus, so they are recognized as *not visible*
        // by the clickable-link fix
        $('.dropdown-toggle a').on('click touchstart', function (e) {
            var $this = $(this);
            // we have to check for visibility because checking for open on the parent does not work
            // the open class is set BEFORE we can check it, so dropdowns are always open at this point.
            if ($this.closest('.dropdown').find('.dropdown-menu').is(':visible')) {
                e.stopPropagation(); // this prevents bootstrap from preventing the click and leads to the browser following the link
            }
        });

        // make a dropdown that is clickable inside itself without closing
        // see https://stackoverflow.com/questions/25089297/twitter-bootstrap-avoid-dropdown-menu-close-on-click-inside for reference
        // handle opening
        $('.dropdown.dropdown-clickable button.dropdown-toggle, .dropdown.dropdown-clickable a.dropdown-toggle').on('click', function () {
            $(this).closest('.dropdown.dropdown-clickable').toggleClass('open');
        });
        // handle closing
        $('body').on('click', function (e) {
            $('.dropdown.dropdown-clickable.open').each(function () {
                var $this = $(this);
                if ($this.has(e.target).length === 0) {
                    $this.removeClass('open');
                }
            });
        });

        // makes arbitrary collapses within the same container behave like an accordeon
        $('.collapse-accordeon .collapse').on('show.bs.collapse', function (e) {
            // prevent collapsing of parents
            e.stopPropagation();
            var $this = $(this);
            // find my own parent accordeon
            var $container = $this.closest('.collapse-accordeon');
            // collapse others except my own parents and myself
            $container.find('.collapse.in').not($this).not($this.parents()).collapse('hide');
        });

        // save to a cookie when someone clicked the gallery or list view buttons
        $('.ed.gallery').click(function () {
            document.cookie = "jtl_shop_ed=gallery; expires=0; path=/";
        });
        $('.ed.list').click(function () {
            document.cookie = "jtl_shop_ed=list; expires=0; path=/";
        });

        // init sliders
        s360_init_sliders();
        $('body').on('article-variation-loaded.s360', s360_init_sliders);

        // initiate shortening for phone
        $('body[data-device="phone"] .shorten-phone').each(function() {
            var that = $(this);
            if(typeof that.data('showCharsPhone') !== 'undefined' && that.data('showCharsPhone') > 0) {
                that.shorten({showChars: that.data('showCharsPhone'), hideLess: true});
            } else if(typeof that.data('showCharsPhone') === 'undefined') {
                that.shorten({hideLess: true});
            }
        });

        // initiate shortening for tablet
        $('body[data-device="tablet"] .shorten-phone').each(function() {
            var that = $(this);
            if(typeof that.data('showCharsTablet') !== 'undefined' && that.data('showCharsTablet')) {
                that.shorten({showChars: that.data('showCharsTablet'), hideLess: true});
            }
        });

        // initiate shortening for desktop
        $('body[data-device="desktop"] .shorten-phone').each(function() {
            var that = $(this);
            if(typeof that.data('showCharsDesktop') !== 'undefined' && that.data('showCharsDesktop')) {
                that.shorten({showChars: that.data('showCharsDesktop'), hideLess: true});
            }
        });

        // get elements that are not links and make them behave like links
        $('body').on('click', '[data-target-url]', function (e) {
            var $this = $(this);
            // only do anything if the real clicked element was not an interactive element itself
            if (!$(e.target).is('a, button, input, select, option, label, .label-variation')
                && !$(e.target).parents('.bootstrap-select').length
                && !$(e.target).parents('.swatches').length
                && !$(e.target).parents('.input-group').length
                && !$(e.target).parents('.lpa-pay-button').length) {
                var url = $this.data('target-url');
                if (url.length) {
                    e.stopPropagation();
                    e.stopImmediatePropagation();
                    window.location.href = url;
                }
            }
        });

        // register slider timer
        $('.s360-image-slider').on('autoChange.s360', function (event, slick, interval) {
            var $slider = slick.$slider;
            var id = $slider.data('slider-id');
            var $progressbar = $('#s360-image-slider-timer-' + id + ' .progress-bar');
            $progressbar.stop();
            $progressbar.css({width: '0', opacity: '1'});
            $progressbar.animate({width: '100%'}, interval, 'linear');
        });
        $('.s360-image-slider').on('autoClear.s360', function (event, slick, interval) {
            var $slider = slick.$slider;
            var id = $slider.data('slider-id');
            var $progressbar = $('#s360-image-slider-timer-' + id + ' .progress-bar');
            $progressbar.stop().animate({opacity: 0}, 250);
        });

        // toggle login@checkout
        $('#registered-customers').change(function () {
            if ($(this).is(":checked")) {
                $('#existing-customer').show();
                return;
            }
            $('#existing-customer').hide();
        });

        // re-initialize slick slider for last viewed articles when the modal is being shown
        $('#recently-viewed-modal').on('shown.bs.modal', function() {
            var $modal = $('#recently-viewed-modal');
            if($modal.find('.slick-slider').length) {
                $modal.find('.slick-slider').slick('refresh');
                _.delay(function() {
                    $modal.find('.s360-article-slider-wrapper').css('visibility', 'visible');
                }, 500);
            }
        });

    });


    $(window).load(function () {
        // adjust autoloaded image boxes on resize or orientation change
        $(window).resize(_.debounce(function () {
            $.evo.imagebox('body');
        }, 100));
    });

})(jQuery);